import os

MLF_FOLDER_NAME = "mlf"
MLRUNS_FOLDER = "mlruns"
MLRUNS_FOLDER_NAME = os.path.join(MLF_FOLDER_NAME, MLRUNS_FOLDER)

MLFOUNDRY_DOCUMENTATION_LINK = "https://truefoundry.gitbook.io/mlfoundry/"
